﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;

namespace TestProject
{
    /// <summary>
    /// 메인 윈도우
    /// </summary>
    public partial class MainWindow : Window
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 포맷 문자열
        /// </summary>
        private const string FORMAT_STRING = "{0,-30} {1,-15} {2,-15} {3,-15}";

        /// <summary>
        /// 마지막 시간
        /// </summary>
        private DateTime lastTime;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainWindow()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();

            string headerString = string.Format
            (
                FORMAT_STRING,
                "Routed Event",
                "sender",
                "Source",
                "OriginalSource"
            );

            this.headerTextBlock.Inlines.Add(new Underline(new Run(headerString)));

            UIElement[] elementArray = { this, this.grid, this.button, this.buttonTextBlock };

            foreach(UIElement element in elementArray)
            {
                // 키보드
                element.PreviewKeyDown   += element_EventHandler;
                element.PreviewKeyUp     += element_EventHandler;
                element.PreviewTextInput += element_EventHandler;
                element.KeyDown          += element_EventHandler;
                element.KeyUp            += element_EventHandler;
                element.TextInput        += element_EventHandler;

                // 마우스
                element.MouseDown        += element_EventHandler;
                element.MouseUp          += element_EventHandler;
                element.PreviewMouseDown += element_EventHandler;
                element.PreviewMouseUp   += element_EventHandler;

                // 스타일러스
                element.StylusDown        += element_EventHandler;
                element.StylusUp          += element_EventHandler;
                element.PreviewStylusDown += element_EventHandler;
                element.PreviewStylusUp   += element_EventHandler;

                element.AddHandler(Button.ClickEvent, new RoutedEventHandler(element_EventHandler));
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 엘리먼트 이벤트 핸들러 처리하기 - element_EventHandler(sender, e)

        /// <summary>
        /// 엘리먼트 이벤트 핸들러 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void element_EventHandler(object sender, RoutedEventArgs e)
        {
            DateTime currentTime = DateTime.Now;

            if(currentTime - this.lastTime > TimeSpan.FromMilliseconds(100))
            {
                this.outputStackPanel.Children.Add(new TextBlock(new Run(" ")));
            }

            this.lastTime = currentTime;

            TextBlock textBlock = new TextBlock();

            textBlock.Text = string.Format
            (
                FORMAT_STRING,
                e.RoutedEvent.Name,
                GetTypeNameWithoutNamespace(sender),
                GetTypeNameWithoutNamespace(e.Source),
                GetTypeNameWithoutNamespace(e.OriginalSource)
            );

            this.outputStackPanel.Children.Add(textBlock);

            (this.outputStackPanel.Parent as ScrollViewer).ScrollToBottom();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 네임스페이스 없는 타입명 구하기 - GetTypeNameWithoutNamespace(source)

        /// <summary>
        /// 네임스페이스 없는 타입명 구하기
        /// </summary>
        /// <param name="source">소스 객체</param>
        /// <returns>네임스페이스 없는 타입명</returns>
        private string GetTypeNameWithoutNamespace(object source)
        {
            string[] itemArray = source.GetType().ToString().Split('.');

            return itemArray[itemArray.Length - 1];
        }

        #endregion
    }
}